"      NAME     UnixTildeSupport
       AUTHOR   barcalow@uiuc.edu, ttang@uiuc.edu
       URL      
       FUNCTION Adds Tilde and Environment Variable support to UnixFilename
       KEYWORDS	
ST-VERSIONS      VisualWorks 2.5
       PREREQUISITES    VisualWorks DLL/C Connect, lib with getpwnam function
       CONFLICTS        
       DISTRIBUTION     world
       VERSION  1.0 (VW2.5)
       DATE     07-Jun-96

SUMMARY

To work correctly, a shared library which contains
the C function
        struct passwd *  getpwnam(const char * )
must be available.  By default this class is setup to
look in /usr/lib/libc.1 (the library on our HP''s).  
This can be changed by editing the ''libraryFiles:'' 
and ''libraryDirectoryies:'' lines in the class 
definition.

For ''~/'' to substitute the current user''s home 
directory, the $USER environment variable must be 
set correctly.

Environment Variable support is an extenation of 
Dr. Kevin Waite's (kww@cs.glasgow.ac.uk) 
EnvVarFileList package

This utility requires VisualWorks 2.5.  A VW 2.0 version
is also available.

                                Jeff Barcalow & Tze-John Tang
"!

'From VisualWorks(R), Release 2.5.1 of September 26, 1995 on April 15, 1996 at 3:41:55 pm'!



ExternalInterface subclass: #UnixTildeExpansion
	includeFiles: ''
	includeDirectories: ''
	libraryFiles: 'libc.1 '
	libraryDirectories: '/usr/lib '
	generateMethods: ''
	beVirtual: false
	optimizationLevel: #full
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'UnixTildeExpansionDictionary '
	category: 'External-Interface-Unix'!
UnixTildeExpansion comment:
'Added by the TildeEnvVarFile package.

UnixTildeExpansion uses the VisualWorks C Connect interface to
lookup a user''s home direcotry in the passwd file.

The interface is the instance-side method ''pwddir:''.

To work correctly, a shared library which contains the C function
	struct passwd *  getpwnam(const char * )
must be available.  By default this class is setup to look in
/usr/lib/libc.1 (the library on our HP''s).  This can be changed
by editing the ''libraryFiles:'' and ''libraryDirectoryies:'' lines in
the class definition.

For ''~/'' to substitute the current user''s home directory, the $USER
environment variable must be set correctly.

Tested under VisualWorks 2.5,  4/14/96

Jeff Barcalow (barcalow@uiuc.edu) and Tze-John Tang (ttang@uiuc.edu)
'!


!UnixTildeExpansion methodsFor: 'structs'!

passwd
	<C: struct passwd {
			char * pw_name;
			char * pw_passwd;
			int pw_uid, pw_gid;
			char * pw_age;
			char * pw_comment;
			char * pw_gecos;
			char * pw_dir;
			char * pw_shell;
			long pw_audid;
			int pw_audflg;
		}>! !

!UnixTildeExpansion methodsFor: 'action'!

pwddir: aUserString
	"takes a string for a username and returns the user's 
	home directory or nil if the username was invalid."

	| passwordEntry charPointer aUS |
	aUS := aUserString.
	"Assume current user's name.  Required $USER to be set."
	(aUS = '')
		ifTrue: [aUS := CEnvironment userEnvironment at: 'USER'].
	passwordEntry := self getpwnam: aUS.
	charPointer := passwordEntry memberAt: #pw_dir.
	^(charPointer contents = 0)
		ifTrue: [nil]
		ifFalse: [charPointer copyCStringFromHeap].! !

!UnixTildeExpansion methodsFor: 'procedures'!

getpwnam: arg
	<C: struct passwd *  getpwnam(const char * )>
	^self externalAccessFailed! !


!UnixFilename class methodsFor: 'private'!

baseDirectoryForList: list
	"Changed by the TildeEnvVarFile package."
	"Based on change in Dr. Kevin Waite's (kww@cs.glasgow.ac.uk) EnvVarFileList package
	ftp://st.cs.uiuc.edu/pub/Smalltalk/MANCHESTER/usenet/st80_pre4/EnvVarFileList.st"

	"'list' is an Ordered Collection of each directory in a 
	directory path.  This method decides whether to
	perform environment variable or tilde substitution by
	checking if the first character in the path is a '~' or '$'"

	^list first = (String with: self separator)
		ifTrue:	[self named: list removeFirst]
		ifFalse:	[|first|
				first := list first first.
				first = $$ 
					ifTrue: [self inLookupEnvironment: list]
					ifFalse: [
						first = $~
							ifTrue: [self inLookupTilde: list]
							ifFalse: [self currentDirectory]]]!

inLookupEnvironment: list
	"Added by the TildeEnvVarFile package."
	"From Dr. Kevin Waite's (kww@cs.glasgow.ac.uk) EnvVarFileList package
	ftp://st.cs.uiuc.edu/pub/Smalltalk/MANCHESTER/usenet/st80_pre4/EnvVarFileList.st"

        "The first member of this list starts with a dollar sign and therefore
        could be a dereferencing of a UNIX environment variable.  Lookup this
        variable in the current environment, and if it is there use its value as
        the directory root, otherwise treat the dollar as an ordinary character."

        | environ key value  root |

        environ := CEnvironment userEnvironment.
        key := list first copyFrom: 2 to: list first size.
        value := environ at: key ifAbsent: [^self currentDirectory].

        root := self named: value.
        (root exists and: [root isDirectory]) ifFalse: [
                ^self currentDirectory].

        "The first part of the given name did correspond to an environment
        variable with a legal directory name as its value.  Use that directory
        as the base directory and strip off the name from the component list."

        list removeFirst.
        ^root!

inLookupTilde: list
	"Added by the TildeEnvVarFile package."
	"The first member of this list starts with a tilde and therefore could
	be a dereferencing of a UNIX user's directory.  Lookup this string
	using UnixTildeExpansion, and if it exists use its value as the base
	directory root, otherwise treat the dollar as an ordinary character."

	| value  root userName |

	userName := list first copyFrom: 2 to: list first size.
	value := (UnixTildeExpansion new pwddir: userName).
	value isNil ifTrue: [^self currentDirectory].
	root := self named: value.
	
	"The username was valid.  Strip off the ~username from the
	component list and return the user's home directory."
	
	list removeFirst. 
	^root!

outLookupEnvironment: str
	"Added by the TildeEnvVarFile package."
	"This message is needed because fileOut doesn't call 'baseDirectoryForList:'"

	"The first member of this list starts with a dollar sign and therefore
	could be a dereferencing of a UNIX environment variable.  Lookup this
	variable in the current environment and if it exists use its value as
	the directory root, otherwise treat the dollar as an ordinary character."

	| environ key value  root list newList |
	list := self components: str.
	environ := CEnvironment userEnvironment.
	key := list first copyFrom: 2 to: list first size.
	value := environ at: key ifAbsent: [^str].

	root := self named: value.
	(root exists and: [root isDirectory]) ifFalse: [^str].

	"The environment variable was valid and refered to a legal directory
	name.  Rebuild a filename string using the user's home directory."

	list removeFirst.
	newList := list inject: value into: [:string :each| string , '/', each].
	^newList!

outLookupTilde: str
	"Added by the TildeEnvVarFile package."
	"This message is needed because fileOut doesn't call 'baseDirectoryForList:'"

	"The first member of this list starts with a tilde and therefore could
	be a dereferencing of a UNIX user's directory. Lookup this string
	using UnixTildeExpansion and if it is there use its value as the
	directory root, otherwise treat the dollar as an ordinary character."

	| value userName list newstr |
	list := self components: str.
	userName := list first copyFrom: 2 to: list first size.
	value := UnixTildeExpansion new pwddir: userName.
	value isNil ifTrue: [^str].

	"The username was valid.  Rebuild the filename string using the
	user's home directory."

	list removeFirst.
	newstr := list inject: value into: [:string :each| string , '/', each].
	^newstr! !




!FileBrowser methodsFor: 'pattern'!

acceptPattern: aText
	"Changed by the TildeEnvVarFile package (VW 2.5)."
      "Change to implement tilde and environment variable substitution for exact
      filename matches."

	"The text consists of file names and file name patterns separated by carriage returns.
	Patterns can include the characters * (any chars) and # (any single char).
	Make my new list be those file names on my directory which match the patterns."

	Cursor wait showWhile:
		[| testPattern newList |
		self pattern: aText string.
		testPattern := self pattern asString.
		newList := SortedCollection forStrings.
		(testPattern includes: $*) | (testPattern includes: $#) |
		   (testPattern includes: $$) | (testPattern includes: $~)
			ifTrue: 
				[Filename errorReporter errorSignal
					handle: [:ex |
							(ex parameter isKindOf: OSErrorHolder)
								ifTrue: [Dialog warn: ex parameter errorString for: self interfaceWindow]
								ifFalse: [Dialog warn: ex errorString for: self interfaceWindow].
							newList := SortedCollection forStrings.
							ex return]
					do: [newList addAll: (self filesMatching: testPattern)]]
			ifFalse: [(self isLegalFileName: testPattern)
						ifTrue: [newList add: (Filename defaultClass canonicalize: testPattern)]].
		self list: newList.
		self changed: #fileNameList.
		self changed: #text.
		^true]! !


!UnixFilename methodsFor: 'private'!

named: n 
	"Changed by the TildeEnvVarFile package (VW 2.5)."
	"Change to implement tilde and environment variable substitution
	during fileOuts."

	"Set the OS-specific pathname."
	"Only appropriate at instance creation."

	| first |
	"Check if first character is '$' (indicating environment variable)
	or '~' (indicating user's home directory)."
	first := n first.
	first = $$ 
		ifTrue: [publicName := self class outLookupEnvironment: n]
		ifFalse: [ (first = $~) ifTrue: [publicName:=self class outLookupTilde: n]
						    ifFalse: [publicName := n]].

	(publicName size > 1 and: [publicName last == self separator and: [publicName ~= '\\']])
		ifTrue: [publicName := publicName copyFrom: 1 to: publicName size - 1].
	osName := IOAccessor convertSimpleStringForPlatform: publicName! !


